#ifndef CUDAUTIL_H
#define CUDAUTIL_H

#include <algorithm>
#include <cstdlib>
#include <cstdio>
#include <vector_types.h>

/**
 * This macro checks return value of the CUDA runtime call and exits
 * the application if the call failed.
 */
#define CUDA_SAFE_CALL(call) {                                             \
    cudaError_t _m_cudaStat = call;                                        \
    if (_m_cudaStat != cudaSuccess) {                                      \
        fprintf(stderr, "Error %s in ""#call"", line %d in file %s\n",     \
                cudaGetErrorString(_m_cudaStat), __LINE__, __FILE__);      \
        exit(1);                                                           \
    }                                                                      \
}

unsigned int inline ceilUintDivision(unsigned int dividend, unsigned int divisor) {
    return (dividend == 0) ? 0 : (1 + (dividend - 1) / divisor);
}







template <typename T>
cudaChannelFormatDesc makeCudaChannelFormatDesc ();

#endif // CUDAUTIL_H
