#ifndef INPUTSTAGE_H
#define INPUTSTAGE_H

#include "stage.h"
#include "ioconfig.h"
#include "processingconfig.h"
#include "imagedimensions.h"
#include "filesystem.h"

template <typename ELEMENT_TYPE>
class InputStage : public Stage
{
public:
    InputStage (const ProcessingConfig & procConfig,
                const IOConfig & ioConfig,
                Stage *next);
    virtual ~InputStage ();
    virtual std::ptrdiff_t startWrite () { return -1; }
    virtual void endWrite (std::ptrdiff_t /* written */ ) {}
    virtual void notifyEndOfStream () {}
    virtual cudaArray* initializeBuffer (unsigned int, unsigned int, unsigned int) { return NULL; }
    virtual cudaArray* initializeBuffer (ImageDimensions const & /* dim */) { return NULL; }

protected:
    virtual void run ();
    void initialize();
private:
    InputStage ();
    IOConfig const ioConfig;
    ProcessingConfig const procConfig;
    ImageDimensions imageDim;
    cudaArray *originalInputArray;
    cudaArray *normalizedFloatArray;
    cudaArray *nextStageArray;
    unsigned char **originalPinnedChannels;
    PathVector inputImages;
};

#endif // INPUTSTAGE_H
