#ifndef PARTICLEFILTERCUDA_H
#define PARTICLEFILTERCUDA_H

#include "imagedimensions.h"
#include "particleset.h"
#include "scalarutils.h"

void launchComputeParticleCoverageMask (const ImageDimensions    & imageDim,
                                        const ParticleManager    & particles,
                                        bool                     * d_coverageMask,
                                        const dim3               & gridSize,
                                        const dim3               & blockSize,
                                        cudaStream_t               stream);

/// Produces a coverage map that
/// shows overlapping particles more easily: each pixel is assigned
///    - 0 if not covered
///    - 1 if covered by an even number of particles
///    - 2 if covered by an odd number of particles
void launchComputeParticleCoverageParity (const ImageDimensions    & imageDim,
                                          const ParticleManager    & particles,
                                          unsigned char            * d_coverageParity,
                                          const dim3               & gridSize,
                                          const dim3               & blockSize,
                                          cudaStream_t               stream);

void launchDetermineCoveredRegions (      unsigned int         numMaxRegions,
                                    const ImageDimensions    & imageDim,
                                    const PixelInRegion      * d_labelledPixels,
                                          unsigned int         numberOfLabelledPixels,
                                    const bool               * d_coverageMask,
                                          unsigned int       * d_coveredRegions,
                                          cudaStream_t         stream);


void launchWeighParticles(const ImageDimensions         & imageDim,
                          float                           likelihoodBeta,
                          const unsigned char           * fgProbability,
                          const PixelInRegion           * d_labelledPixels,
                                unsigned int              numberOfLabelledPixels,
                                ParticleManager         & particles,
                                cudaStream_t              stream);

void launchComputeParticleFGPriorProbability (const ImageDimensions    & imageDim,
                                       const ParticleManager    & particles,
                                             float              * d_probability,
                                       const dim3               & gridSize,
                                       const dim3               & blockSize,
                                       cudaStream_t               stream);

void launchComputeConstantFGPriorProbability(const ImageDimensions & imageDim,
                                                   float           * d_probability,
                                             const dim3            & gridSize,
                                             const dim3            & blockSize,
                                                   cudaStream_t      stream);

#endif // PARTICLEFILTERCUDA_H
